@@ -29,6 +29,7 @@
 #include "pango-engine.h"
 #include "pango-engine-private.h"
 #include "pango-modules.h"
+#include "pango-script-private.h"
 
 struct _PangoContext
 {
@@ -286,7 +287,6 @@ pango_context_load_font (PangoContext               *context,
 			 const PangoFontDescription *desc)
 {
   g_return_val_if_fail (context != NULL, NULL);
-  g_return_val_if_fail (pango_font_description_get_family (desc) != NULL, NULL);
 
   return pango_font_map_load_font (context->font_map, context, desc);
 }
@@ -305,11 +305,9 @@ pango_context_load_font (PangoContext               *context,
 PangoFontset *
 pango_context_load_fontset (PangoContext               *context,
 			    const PangoFontDescription *desc,
-			     PangoLanguage             *language)
+			    PangoLanguage             *language)
 {
   g_return_val_if_fail (context != NULL, NULL);
-  g_return_val_if_fail (pango_font_description_get_family (desc) != NULL, NULL);
-  g_return_val_if_fail (pango_font_description_get_size (desc) != 0, NULL);
 
   return pango_font_map_load_fontset (context->font_map, context, desc, language);
 }
@@ -685,7 +683,7 @@ struct _ItemizeState
 
   ChangedFlags changed;
 
-  PangoScriptIter *script_iter;
+  PangoScriptIter script_iter;
   const char *script_end;
   PangoScript script;
 
@@ -850,8 +848,8 @@ itemize_state_init (ItemizeState      *state,
 
   /* Initialize the script iterator
    */
-  state->script_iter = pango_script_iter_new (text + start_index, length);
-  pango_script_iter_get_range (state->script_iter, NULL,
+  _pango_script_iter_init (&state->script_iter, text + start_index, length);
+  pango_script_iter_get_range (&state->script_iter, NULL,
 			       &state->script_end, &state->script);
 
   update_end (state);
@@ -899,8 +897,8 @@ itemize_state_next (ItemizeState *state)
 
   if (state->run_end == state->script_end)
     {
-      pango_script_iter_next (state->script_iter);
-      pango_script_iter_get_range (state->script_iter, NULL,
+      pango_script_iter_next (&state->script_iter);
+      pango_script_iter_get_range (&state->script_iter, NULL,
 				   &state->script_end, &state->script);
       state->changed |= SCRIPT_CHANGED;
     }
@@ -1067,6 +1065,9 @@ get_shaper_and_font_foreach (PangoFontset *fontset G_GNUC_UNUSED,
   GetShaperFontInfo *info = data;
   GSList *l;
 
+  if (G_UNLIKELY (!font))
+    return FALSE;
+
   for (l = info->engines; l; l = l->next)
     {
       PangoEngineShape *engine = l->data;
@@ -1407,7 +1408,7 @@ itemize_state_finish (ItemizeState *state)
   g_free (state->embedding_levels);
   if (state->free_attr_iter)
     pango_attr_iterator_destroy (state->attr_iter);
-  pango_script_iter_free (state->script_iter);
+  _pango_script_iter_fini (&state->script_iter);
   pango_font_description_free (state->font_desc);
 
   itemize_state_reset_shape_engines (state);
