@@ -618,13 +618,15 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
   PangoWin32Family *win32family;
   PangoFont *result = NULL;
   GSList *tmp_list;
+  const char *family;
 
   g_return_val_if_fail (description != NULL, NULL);
 
-  PING (("name=%s", pango_font_description_get_family (description)));
+  family = pango_font_description_get_family (description);
+  family = family ? family : "";
+  PING (("name=%s", family))
 
-  win32family = g_hash_table_lookup (win32fontmap->families,
-				     pango_font_description_get_family (description));
+  win32family = g_hash_table_lookup (win32fontmap->families, family);
   if (win32family)
     {
       PangoWin32Face *best_match = NULL;
@@ -667,8 +669,9 @@ pango_win32_font_neww (PangoFontMap   *fontmap,
 
   result = (PangoWin32Font *)g_object_new (PANGO_TYPE_WIN32_FONT, NULL);
 
+  g_assert (result->fontmap == NULL);
   result->fontmap = fontmap;
-  g_object_ref (fontmap);
+  g_object_add_weak_pointer (G_OBJECT (result->fontmap), (gpointer *) (gpointer) &result->fontmap);
 
   result->size = size;
   _pango_win32_make_matching_logfontw (fontmap, lfp, size, &result->logfontw);
@@ -713,7 +716,6 @@ pango_win32_font_map_real_find_font (PangoWin32FontMap          *win32fontmap,
   if (!win32font)
     return NULL;
 
-  win32font->fontmap = fontmap;
   win32font->win32face = face;
   face->cached_fonts = g_slist_prepend (face->cached_fonts, win32font);
 
@@ -1069,63 +1071,9 @@ pango_win32_font_description_from_logfontw (const LOGFONTW *lfp)
   gchar *family;
   PangoStyle style;
   PangoVariant variant;
-  PangoWeight weight, name_weight;
+  PangoWeight weight;
   PangoStretch stretch;
 
-  static const struct {
-    const char *marker;
-    int marker_len;
-    int remove_len;
-    PangoWeight weight;
-  } weight_names[] = {
-#define ENTRY(n, s) ENTRY2 (n, sizeof (#n) - 1, s)
-#define ENTRY2(n, l, s) ENTRY3 (n, l, l, s)
-#define ENTRY3(n, marker_len, remove_len, s) { #n, marker_len, remove_len, PANGO_WEIGHT_##s }
-    ENTRY (Ultra Light, ULTRALIGHT),
-    ENTRY (UltraLight, ULTRALIGHT),
-    ENTRY (Light, LIGHT),
-    ENTRY (Medium, NORMAL),
-    ENTRY (Demi Bold, SEMIBOLD),
-    ENTRY (Demi, SEMIBOLD),
-    ENTRY (Ultra Bold, ULTRABOLD),
-    ENTRY (Extra Bold, ULTRABOLD),
-    ENTRY (SemiBold, SEMIBOLD),
-    ENTRY (DemiBold, SEMIBOLD),
-    ENTRY (UltraBold, ULTRABOLD),
-    ENTRY (ExtraBold, ULTRABOLD),
-    ENTRY (Bold, BOLD),
-    ENTRY (Heavy, HEAVY),
-    ENTRY (Black, HEAVY),
-#undef ENTRY
-#undef ENTRY2
-#undef ENTRY3
-  };
-
-  static const struct {
-    const char *marker;
-    int marker_len;
-    PangoStretch stretch;
-  } stretch_names[] = {
-#define ENTRY(n, s) { #n, sizeof (#n) - 1, PANGO_STRETCH_##s }
-    ENTRY (Ext Condensed, EXTRA_CONDENSED),
-    ENTRY (Extra Condensed, EXTRA_CONDENSED),
-    ENTRY (UltraCondensed, ULTRA_CONDENSED),
-    ENTRY (ExtraCondensed, EXTRA_CONDENSED),
-    ENTRY (Condensed, CONDENSED),
-    ENTRY (Cond, CONDENSED),
-    ENTRY (Narrow, CONDENSED),
-    ENTRY (Ext Expanded, EXTRA_EXPANDED),
-    ENTRY (Extra Expanded, EXTRA_EXPANDED),
-    ENTRY (Ultra Expanded, ULTRA_EXPANDED),
-    ENTRY (ExtraExpanded, EXTRA_EXPANDED),
-    ENTRY (UltraExpanded, ULTRA_EXPANDED),
-    ENTRY (Expanded, EXPANDED),
-#undef ENTRY
-  };
-
-  int i;
-  char *p;
-
   family = get_family_nameW (lfp);
 
   if ((lfp->lfPitchAndFamily & 0xF0) == FF_ROMAN && lfp->lfItalic)
@@ -1158,51 +1106,9 @@ pango_win32_font_description_from_logfontw (const LOGFONTW *lfp)
   else
     weight = PANGO_WEIGHT_HEAVY;
 
-  name_weight = 0;
-
-  p = family;
-  while ((p = strchr (p, ' ')) != NULL)
-    {
-      for (i = 0; i < G_N_ELEMENTS (weight_names); i++)
-	{
-	  if (g_ascii_strncasecmp (p + 1, weight_names[i].marker, weight_names[i].marker_len) == 0 &&
-	      (p[1 + weight_names[i].marker_len] == '\0' ||
-	       p[1 + weight_names[i].marker_len] == ' '))
-	    {
-	      strcpy (p, p + 1 + weight_names[i].remove_len);
-	      name_weight = weight_names[i].weight;
-	      break;
-	    }
-	}
-      if (i < G_N_ELEMENTS (weight_names))
-	break;
-      p++;
-    }
-
-  if (weight == PANGO_WEIGHT_NORMAL && name_weight > 0)
-    weight = name_weight;
-
+  /* XXX No idea how to figure out the stretch */
   stretch = PANGO_STRETCH_NORMAL;
 
-  p = family;
-  while ((p = strchr (p, ' ')) != NULL)
-    {
-      for (i = 0; i < G_N_ELEMENTS (stretch_names); i++)
-	{
-	  if (g_ascii_strncasecmp (p + 1, stretch_names[i].marker, stretch_names[i].marker_len) == 0 &&
-	      (p[1 + stretch_names[i].marker_len] == '\0' ||
-	       p[1 + stretch_names[i].marker_len] == ' '))
-	    {
-	      strcpy (p, p + 1 + stretch_names[i].marker_len);
-	      stretch = stretch_names[i].stretch;
-	      break;
-	    }
-	}
-      if (i < G_N_ELEMENTS (stretch_names))
-	break;
-      p++;
-    }
-
   description = pango_font_description_new ();
   pango_font_description_set_family (description, family);
   g_free(family);
